@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)recipient.c	8.154 (Berkeley) 6/24/98";
+static char sccsid[] = "@(#)recipient.c	8.161 (Berkeley) 12/18/1998";
 #endif /* not lint */
 
 # include "sendmail.h"
@@ -331,8 +331,11 @@ recipient(a, sendq, aliaslevel, e)
 	}
 
 	/* add address on list */
-	*pq = a;
-	a->q_next = NULL;
+	if (pq != NULL)
+	{
+		*pq = a;
+		a->q_next = NULL;
+	}
 
 	/*
 	**  Alias the name and handle special mailer types.
@@ -340,7 +343,10 @@ recipient(a, sendq, aliaslevel, e)
 
   trylocaluser:
 	if (tTd(29, 7))
-		printf("at trylocaluser %s\n", a->q_user);
+	{
+		printf("at trylocaluser: ");
+		printaddr(a, FALSE);
+	}
 
 	if (bitset(QDONTSEND|QBADADDR|QVERIFIED, a->q_flags))
 		goto testselfdestruct;
@@ -931,7 +937,7 @@ include(fname, forwarding, ctladdr, sendq, aliaslevel, e)
 		       (int) getuid(), (int) geteuid());
 
 	if (forwarding)
-		sfflags |= SFF_MUSTOWN|SFF_ROOTOK|SFF_NOSLINK;
+		sfflags |= SFF_MUSTOWN|SFF_ROOTOK|SFF_NOWLINK;
 
 	ca = getctladdr(ctladdr);
 	if (ca == NULL)
@@ -1195,7 +1201,7 @@ resetuid:
 	}
 
 	/*
-	** Check to see if some bad guy can write this file
+	**  Check to see if some bad guy can write this file
 	**
 	**	Group write checking could be more clever, e.g.,
 	**	guessing as to which groups are actually safe ("sys"
